#include <Game.h>
#include <Category.h>
#include <Entity.h>

#include <StringHelpers.h>
#include <iostream>

#include <math.h> 

const sf::Time Game::TimePerFrame = sf::seconds(1.f/60.f);
const int Game::Width = 800;
const int Game::Height = 600;
const float StartOffset = 50.f;

Game::Game()
: mWindow(sf::VideoMode(Game::Width, Game::Height), "SFML Application", sf::Style::Close)
, mTextures()
, mFonts()
, mEntities()
, mStatisticsText()
, mStatisticsUpdateTime()
, mStatisticsNumFrames(0)
, mIsMovingUp(false)
, mIsMovingDown(false)
, mIsMovingRight(false)
, mIsMovingLeft(false)
{
	mTextures.load(Textures::BlueCar, "resources/Textures/SimpleBlueCarTopView-300px.png");
	mTextures.load(Textures::GreenCar, "resources/Textures/SimpleGreenCarTopView-300px.png");
	mTextures.load(Textures::RedCar, "resources/Textures/SimpleRedCarTopView-300px.png");
	mTextures.load(Textures::YellowCar, "resources/Textures/SimpleYellowCarTopView-300px.png");
	
	mEntities.push_back(std::move(std::make_unique<Entity>(mTextures, Textures::BlueCar, Category::PlayerCar, StartOffset, StartOffset, 45.f)));
	mEntities.push_back(std::move(std::make_unique<Entity>(mTextures, Textures::GreenCar, Category::NonPlayerCar, Width-StartOffset, StartOffset, 135.f)));
	mEntities.push_back(std::move(std::make_unique<Entity>(mTextures, Textures::RedCar, Category::NonPlayerCar, Width-StartOffset, Height-StartOffset, 225.f)));
	mEntities.push_back(std::move(std::make_unique<Entity>(mTextures, Textures::YellowCar, Category::NonPlayerCar, StartOffset, Height - StartOffset, -45.f)));

	mPlayerNum = 0;

	mFonts.load(Fonts::Main, "resources/Sansation.ttf");
	mStatisticsText.setFont(mFonts.get(Fonts::Main));
	mStatisticsText.setPosition(5.f, 5.f);
	mStatisticsText.setCharacterSize(10);
}

void Game::run()
{
	sf::Clock clock;
	sf::Time timeSinceLastUpdate = sf::Time::Zero;
	while (mWindow.isOpen())
	{
		sf::Time elapsedTime = clock.restart();
		timeSinceLastUpdate += elapsedTime;
		while (timeSinceLastUpdate > TimePerFrame)
		{
			timeSinceLastUpdate -= TimePerFrame;

			processEvents();
			update(TimePerFrame);
		}

		updateStatistics(elapsedTime);
		render();
	}
}

void Game::processEvents()
{
	sf::Event event;
	while (mWindow.pollEvent(event))
	{
		switch (event.type)
		{
			//case sf::Event::KeyPressed:
			//	handlePlayerInput(event.key.code, true);
			//	break;

			case sf::Event::KeyReleased:
				handlePlayerInput(mPlayerNum, event.key.code);
				break;

			case sf::Event::Closed:
				mWindow.close();
				break;
		}
	}
}

void Game::update(sf::Time elapsedTime)
{
	for (auto &entity : mEntities) {
		entity->update(elapsedTime, Width, Height);
	}
}

void Game::render()
{
	mWindow.clear();
	for (auto const &entity : mEntities) {
		mWindow.draw(*entity);
	}
	mWindow.draw(mStatisticsText);
	mWindow.display();
}

void Game::updateStatistics(sf::Time elapsedTime)
{
	mStatisticsUpdateTime += elapsedTime;
	mStatisticsNumFrames += 1;

	if (mStatisticsUpdateTime >= sf::seconds(1.0f))
	{
		mStatisticsText.setString(
			"Frames / Second = " + toString(mStatisticsNumFrames) + "\n" +
			"Time / Update = " + toString(mStatisticsUpdateTime.asMicroseconds() / mStatisticsNumFrames) + "us");

		mStatisticsUpdateTime -= sf::seconds(1.0f);
		mStatisticsNumFrames = 0;
	}
}

void Game::handlePlayerInput(int playerNumber, sf::Keyboard::Key key)
{
	if (key == sf::Keyboard::Up) {
		mEntities[playerNumber]->speedFactor(+1);
	}
	else if (key == sf::Keyboard::Down) {
		mEntities[playerNumber]->speedFactor(-1);
	}
	else if (key == sf::Keyboard::Left) {
		mEntities[playerNumber]->wheelsTurn(-1);
	}
	else if (key == sf::Keyboard::Right) {
		mEntities[playerNumber]->wheelsTurn(+1);
	}
	std::cout << mEntities[playerNumber]->getSpeedFactor() << " / " << mEntities[playerNumber]->getWheelsTurn() << std::endl;

}
